UMDConstructor(
    this,
    'FetchConfigUtils',
    {
        'Authentication': typeof require !== 'undefined' && require('../../Authentication/authentication.es5')
    },
    function(Authentication) {
        function getCMCConfig(){
            var CMCconfig;
            CMCconfig = xhrGET( 'assets/CMC.config.json?t='+ new Date().getTime()  );
            return CMCconfig;
        }

        function getEncrypeConfig(apiServeAddr) {
          var promise = new Promise((resolve, reject) => {
            console.log("getEncrypeConfig=====", apiServeAddr);
            if(apiServeAddr) {
              let reqUrl = apiServeAddr + 'api/Configuration/GetEncrypeConfig';
              var xhr = new XMLHttpRequest();
              xhr.open('get', reqUrl, true);
              xhr.responseType = 'text';
              xhr.onreadystatechange = (event) => {
                if (xhr.readyState === 4) {
                  if (xhr.status === 200) {
                    resolve(xhr.responseText);
                  }else{
                    var errorReason = "";
                    if (xhr.status === 0) {
                        errorReason = "连接服务器失败，请检查网络和配置";
                    } else if(xhr.status === 404) {
                        errorReason = "配置不存在，请联系管理员";
                    } else if(xhr.status === 503) {
                        errorReason = "服务异常，请稍候重试，如仍有问题请联系管理员";
                    }
                    var errorDesc = '加载失败，' + errorReason + ',url:' + reqUrl+',code:' + xhr.status + ',response:' + xhr.responseText;
                    console.error(errorDesc);
  
                    resolve(xhr.responseText);
                  }
                }
              }
              xhr.send();
            }
          });
          return promise;
        }

        function getOrgServeAddr(centerServeAddr, projectId){
            var rsp = FetchConfigUtils.xhrGET( centerServeAddr + 'api/Organization/getV2/' + projectId,undefined,undefined,true );

            if( rsp && rsp.errcode === "SUCCESS" ) {
                var orgServeInfo = rsp.data || {};
                // if(!orgServeInfo.ProjectAPIUrl){
                //     orgServeInfo.ProjectAPIUrl = 'https://yycx-api.sschospital.cn/consultationapi-test/';
                // }
                return orgServeInfo.ProjectAPIUrl;
            }
            return null;
        }
    
        // 预设服务地址
        function preSetServeUrl(projectid, webApi, serveUrl ) {
            // 预设服务地址
            try{
                var webApiObj = webApi && JSON.parse( webApi ) || {};
                webApiObj.businessUrl = serveUrl; // NOTE: 不能以手动配置为由，否则永久得不到更新
                webApiObj.APINetType = 0;
                webApiObj.projectId = projectid;
                localStorage.setItem('webApi', JSON.stringify( webApiObj ) ); 
            } catch( err ) {
                console.error('preSetServeUrl:', err );
            }
        }

        function xhrRequest( method, reqUrl, errCallback, successCallBack, isEncryption) {
            var resData;
            
            try {
                resData = {};
        
                var xhr = new XMLHttpRequest();
                xhr.open(method, reqUrl, false);

                try{
                    if( isEncryption ){
                        var nonce = Authentication.randomString(8);
                        var timestamp = new Date().getTime().toString();
                        var sign = Authentication.encryption(timestamp,nonce);
                        xhr.setRequestHeader('sign', sign);
                        xhr.setRequestHeader('timestamp', timestamp);
                        xhr.setRequestHeader('nonce', nonce);
                    }
                }catch (error){
                    console.error('xhr setRequestHeader err:', error);
                }

                try {
                    xhr.send();
                } catch (error) {
                    var errorDesc = '连接失败,url:' + reqUrl;
                    console.error('xhr send err:', errorDesc);
                    errCallback && errCallback();
                    return;
                }
                
                if (xhr.readyState === 4) {
                    if (xhr.status === 200 || xhr.status ===304) {
                        try {
                            resData = JSON.parse(xhr.responseText);
                            successCallBack && successCallBack( resData );
                        } catch (error) {
                            console.error('解析数据失败，返回的不是正常的JSON数据，url:' + reqUrl + ',responseText:' + xhr.responseText);
                            errCallback && errCallback();
                            return;
                        }
                    }else{
                        var errorReason = "";
                        if (xhr.status === 0) {
                            errorReason = "连接服务器失败，请检查网络和配置";
                        } else if(xhr.status === 404) {
                            errorReason = "配置不存在，请联系管理员";
                        } else if(xhr.status === 503) {
                            errorReason = "服务异常，请稍候重试，如仍有问题请联系管理员";
                        }
                        var errorDesc = '加载失败，' + errorReason + ',url:' + reqUrl+',code:' + xhr.status + ',response:' + xhr.responseText;
                        console.error(errorDesc);

                        try {
                            resData = JSON.parse(xhr.responseText);
                        } catch (error) {
                            console.error('解析数据失败，返回的不是正常的JSON数据，url:' + reqUrl + ',responseText:' + xhr.responseText);
                        }
                        errCallback && errCallback( resData );
                    }
                }
            } catch (error) {
                console.error('发生未处理的异常',error);
                errCallback && errCallback();
            }
            return resData;
        }

        function xhrPOST(reqUrl, errCallback, successCallBack, isEncryption) {
            return xhrRequest( 'post', reqUrl, errCallback, successCallBack, isEncryption)
        }
    
        function xhrGET( reqUrl, errCallback, successCallBack, isEncryption ) {
            return xhrRequest( 'get', reqUrl, errCallback, successCallBack, isEncryption)
        }
    
        var FetchConfigUtils = {
            getCMCConfig: getCMCConfig,
            getEncrypeConfig: getEncrypeConfig,
            preSetServeUrl: preSetServeUrl,
            xhrGET: xhrGET,
            xhrPOST: xhrPOST,
            getOrgServeAddr: getOrgServeAddr,
        };

        return FetchConfigUtils;
    }
);

// 支持模块化引用
function UMDConstructor(root, exportName, depsMap, factoryFn ) {
    var depPathsRequireds = Object.keys(depsMap).map(function(key) { // NOTE: 企业微信PC端并未支持Object.values
        return depsMap[key];
    });
    var depExportNames = Object.keys( depsMap );
    var getDepsExports = function() {
        return depExportNames.map( function(moduleExportName) { 
            var moduleObj = root[moduleExportName];
            
            if( moduleObj ) {
                return moduleObj
            }
            throw new Error('Cannot find module \'' + moduleExportName + '\'');
         });
    }

    if (typeof module === 'object' && typeof module.exports === 'object') { // commonjs模块规范，nodejs环境
        module.exports = factoryFn.apply(null, depPathsRequireds);
    } else if (typeof define === 'function' && define.amd) { // AMD模块规范，require.js
        define(depPaths, factoryFn);
    } else if (typeof define === 'function' && define.cmd) {  // CMD模块规范，sea.js
        define(function(require, exports, module) {
            module.exports = factoryFn.apply(null, depPathsRequireds);
        })
    } else {
        root[exportName] = factoryFn.apply(null, getDepsExports()); // 没有模块环境，直接挂载在全局对象上
    }
}