UMDConstructor(
    this,
    'LoggerUtils',
    {
    },
    function() {
        var ConsoleLogLevel = {
            DEBUG: 1,
            INFO: 2,
            WARN: 3,
            ERROR: 4
        };
        var webApi = JSON.parse( localStorage.getItem('webApi') ) || {};
        var globalUserInfo = JSON.parse( localStorage.getItem('globalUserInfo') ) || {};
        globalUserInfo.userInfo = {
            UserName: globalUserInfo.UserName,
            Name: globalUserInfo.Name,
        };
        
        var LoggerUtils = {
            platform: undefined,
            AppService: undefined,
            globalUserInfo: globalUserInfo,
            logQueue: [],
            ConsoleLogLevel: ConsoleLogLevel,
            webApi: webApi,
            init: function() {
                var console = window.console;
    
                if (typeof window.logInited != 'undefined') {
                    console.log('日志已进行初始化，不重复初始化');
                    return;
                }
    
                window.logInited = true;
                this.sendLogs();
    
                console.level = this.ConsoleLogLevel.DEBUG;
                var logger = this;
                var that = this;
                console.defaultLog = console.log.bind(console);
                console.log = function () {
                    if (console.level === ConsoleLogLevel.DEBUG) {
                        console.defaultLog.apply(console, arguments);
                    }
                }
    
                console.defaultDebug = console.debug.bind(console);
                console.debug = function () {
                    if (console.level === ConsoleLogLevel.DEBUG) {
                        console.defaultDebug.apply(console, arguments);
                    }
                }
    
                console.defaultInfo = console.info.bind(console);
                console.info = function () {
                    if (console.level <= ConsoleLogLevel.INFO) {
                        console.defaultInfo.apply(console, arguments);
                    }
                }
    
                console.defaultWarn = console.warn.bind(console);
                console.warn = function () {
                    if (console.level <= ConsoleLogLevel.WARN) {
                        console.defaultWarn.apply(console, arguments);
                    }
                    var level = ConsoleLogLevel.WARN;
                    that.sendToServer(level,Array.from(arguments));
                }
    
                console.defaultError = console.error.bind(console);
                console.error = function () {
                    if (console.level <= ConsoleLogLevel.ERROR) {
                        try{
                            console.defaultError.apply(console, arguments);
                        } catch( e ) {
                            console.defaultWarn('WinConsole.defaultError.apply err:', e);
                        }
                    }
                    var level = ConsoleLogLevel.ERROR;
                    that.sendToServer(level,Array.from(arguments));
                }
    
                window.onerror = function (msg, url, lineNo, columnNo, error) {
                    window.console.log('window.onerror');
                    
                    if(0) { // TODO: 判断网络断开时,防止循环触发onerror【目前未发现】; 当前确认：可捕获脚本语法错误
                        return;
                    }
    
                    try {
                        var message = '发生未处理的异常：';
                        if (typeof (msg) !== 'string') {
                            var datas = Array.from(arguments);
                            datas.unshift(message);
                            var level = ConsoleLogLevel.ERROR;
                            logger.sendToServer(level, datas);
                            return;
                        }
                        var string = msg.toLowerCase();
                        var substring = "script error";
                        if (string.indexOf(substring) > -1) {
                            //console.error('Script Error: See Browser Console for Detail');
                        } else {
                            var device = {
                              "Time": new Date()
                            };
    
                            var data = {
                                message: message + msg,
                                scriptUrl: url,
                                stackTrace: error && error.stack,
                                user: logger.globalUserInfo && (logger.globalUserInfo.userInfo !== undefined ? logger.globalUserInfo.userInfo.UserName : '-'),
                                appType: 'app',
                                platform: logger.platform && logger.platform.platforms().join(','),
                                projectId: webApi.projectId,
                                device: device
                            };
                            logger.addToLogQueue(data);
                        }
    
                        
                    } catch (error) {
                        var console = window.console;
                        console.defaultError('处理未处理的异常时发生错误:', error);
                    }
                    return false;
    
                };
                return this;
            },
            sendToServer: function(level,logParams) {
            
                try {
                    var device = {
                      "Time": new Date()
                    };
                    if (logParams.length > 0) {
                        var message='';
                        for (var para of logParams) {
                            if (para===null) {
                                message+=' null ';
                            }
                            else if (para===undefined) {
                                message+=' undefined ';
                            }
                            else if (typeof(para)!=='object') {
                                message+=para;
                                if (!message.endsWith(' ')) {
                                    message+=' ';
                                }
                            }
                            else if(para instanceof Error){
                                message+=para.message;
                            }
                            else {
                                if (message.indexOf('ERROR CONTEXT')!==-1) {
                                    return;
                                }
                                var copyedPra;
                                try {
                                    copyedPra = JSON.parse(this.stringifyLimitLength(para,{cycles: true}));
                                } catch (error) {
                                    copyedPra = '数据序列化错误：' + error.message + ',数据类型：' + typeof para;
                                }
        
                                try {
                                    delete copyedPra.promise;
                                } catch (para) {
                                    
                                }
                                try {
                                    delete copyedPra.zone;
                                } catch (error) {
                                    
                                }
                                try {
                                    delete copyedPra.task;
                                } catch (error) {
                                    
                                }
                                message+='\r\n';
                                message+=this.stringify(copyedPra);
                                message+='\r\n';
                            }
                        }
                        var error = logParams.find(function (item){ return item instanceof Error});
                        var data = {
                            level: level,
                            message: message,
                            scriptUrl: '',
                            stackTrace: error!==undefined? error.stack : '-',
                            user: this.globalUserInfo && ( this.globalUserInfo.userInfo!==undefined? this.globalUserInfo.userInfo.UserName + '(' + this.globalUserInfo.userInfo.Name + ')' : '-' ),
                            appType: 'app',
                            platform: this.platform && this.platform.platforms().join(','),
                            projectId: webApi.projectId,
                            device: device,
                            data: []
                        };
                        this.addToLogQueue(data);
                    }
                } catch (error) {
                    var console = window.console;
                    console.defaultError('日志发送失败,发生错误:',error);
                }
                
            },
            sendLogs: function() {
                var that = this;
                setTimeout(function() {
                    var checkIn5Sec = function () {
                        var logData = that.logQueue.shift();
                        
                        if (!logData) {
                            that.delay(5000, function() {
                                checkIn5Sec();
                            });
                        } else {
                            sendReq( logData );
                        }
                    }
    
                    var checkIn20Sec = function ( logData ) {
                        that.logQueue.unshift(logData);
                        that.delay(20000, function() {
                            checkIn5Sec();
                        });
                    }
    
                    var sendReq = function ( logData ) {
                        try {
                            that.sendRequest(logData, function(isSuccess) {
                                if( isSuccess ) {
                                    checkIn5Sec();
                                } else {
                                    checkIn20Sec( logData );
                                }
                            });
                        } catch (error) {
                            checkIn20Sec( logData );
                        }
                    }
    
                    checkIn5Sec();
                   
                }, 0);
            },
            delay: function( timespan, callback ) {
                setTimeout(function() {
                    callback( true );
                }, timespan);
            },
            addToLogQueue: function(data){
                if (!data) {
                    return;
                }
                if (this.logQueue.length>100) {
                    var console = window.console;
                    console.defaultWarn( '日志队列已满(' + this.logQueue.length + '条)，不再添加');
                    return;
                }
                this.logQueue.push(data);
            },
            sendRequest: function(data, callback, addToDemoGraylog, logContent) {
                try {
                    var baseUrl = undefined;
                    addToDemoGraylog = addToDemoGraylog == undefined ? false : addToDemoGraylog;
                    logContent = logContent == undefined ? this.stringify(data) : logContent;
                    var that = this;
                    var console = window.console;
                    if (!webApi || !webApi.businessUrl || addToDemoGraylog) {
                        try {
                          !addToDemoGraylog && console.defaultError('可能还未初始化businessUrl，所以使用演示地址发送日志');
                        } catch (error) {
                            
                        }
                        baseUrl = "https://cmc.yunyi.pro/consultationapi/";
                    }else{
                        baseUrl = webApi.businessUrl;
                    }
                    if (!baseUrl.endsWith('/')) {
                        baseUrl+='/';
                    }
                    var url = baseUrl + "api/logger";
                    var xhr = new XMLHttpRequest();
                    xhr.timeout = 10000;
                    xhr.open("POST", url, true);
                    var headers={};
                    headers['Content-Type'] = 'application/json';
                    // TODO: 使用Token并解密sm4情况
                    Object.keys(headers).forEach(function(key) {
                        xhr.setRequestHeader(key, headers[key]);
                    });
                    xhr.onreadystatechange = function(event) {
                        if (xhr.readyState === 4) {
                          
                            if (xhr.status === 200) {
                                callback(true, xhr.response );
                            }else if( !addToDemoGraylog && xhr.status === 401 ) {
                              console.defaultError('日志发送失败,尝试上传至演示graylog,status:'+ xhr.status +',response:' + xhr.response);
                              that.sendRequest(null,callback, true, logContent);
                            } else {
                              console.defaultError('日志发送失败,status:' + xhr.status + ',response:' + xhr.response);
                              callback(false, xhr );
                            }
                        }
                    };
    
                    xhr.send( logContent );
                } catch (error) {
                    var console = window.console;
                    console.defaultError('日志发送出错', error);
                    callback(false, error );
                }
            },
            stringifyLimitLength: function(data, opts) {
                if (!opts) opts = {};
                if (typeof opts === 'function') opts = { cmp: opts };
                var cycles = (typeof opts.cycles === 'boolean') ? opts.cycles : false;
            
                var cmp = opts.cmp && (function (f) {
                    return function (node) {
                        return function (a, b) {
                            var aobj = { key: a, value: node[a] };
                            var bobj = { key: b, value: node[b] };
                            return f(aobj, bobj);
                        };
                    };
                })(opts.cmp);
            
                var seen = [];
                var logger = this;

                return (function stringify (node) {
                    if (node && node.toJSON && typeof node.toJSON === 'function') {
                        node = node.toJSON();
                    }
            
                    if (node === undefined) return;
                    if (typeof node == 'number') return isFinite(node) ? '' + node : 'null';
                    if (typeof node !== 'object') return JSON.stringify(node);
            
                    var i, out;
                    if (Array.isArray(node)) {
                        out = '[';
                        // for (i = 0; i < node.length; i++) {
                        //     if (i) out += ',';
                        //     out += logger.stringify(node[i]) || 'null';
                        // }
                        out += 'Array.(Not supported at the moment)';
                        return out + ']';
                    }
            
                    if (node === null) return 'null';
            
                    if (seen.indexOf(node) !== -1) {
                        if (cycles) return JSON.stringify('__cycle__');
                        throw new TypeError('Converting circular structure to JSON');
                    }
            
                    var seenIndex = seen.push(node) - 1;
                    var keys = Object.keys(node).sort(cmp && cmp(node));
                    out = '';
                    for (i = 0; i < keys.length; i++) {
                        var key = keys[i];
                        var value = logger.stringify(node[key]);
                        var value = 'item.(Not supported at the moment)';
            
                        if (!value) continue;
                        if (out) out += ',';
                        out += JSON.stringify(key) + ':' + value;
                        if (out.length>5000) {
                            if (out) out += ',';
                            out += '"error": "数据过长"';
                            break;
                        }
                    }
                    seen.splice(seenIndex, 1);
                    return '{' + out + '}';
                })(data);
            },
            stringify: function(obj, replacer, spaces, cycleReplacer = null) {
                var stack = [], keys = [];
                return JSON.stringify(obj, this.serializer(replacer, cycleReplacer, stack, keys), spaces)
            },
            serializer: function(replacer, cycleReplacer, stack, keys) {

                if (cycleReplacer === null || cycleReplacer === undefined) cycleReplacer = function (key, value) {
                    if (stack[0] === value) return "[Circular ~]"
                    return "[Circular ~." + keys.slice(0, stack.indexOf(value)).join(".") + "]"
                }
        
                return function (key, value) {
                    if (stack.length > 0) {
                        var thisPos = stack.indexOf(this)
                        ~thisPos ? stack.splice(thisPos + 1) : stack.push(this)
                        ~thisPos ? keys.splice(thisPos, Infinity, key) : keys.push(key)
                        if (~stack.indexOf(value)) value = cycleReplacer.call(this, key, value)
                    }
                    else stack.push(value)
        
                    return replacer == null ? value : replacer.call(this, key, value)
                }
            }
        };
        
        return LoggerUtils;
    }
);

// 支持模块化引用
function UMDConstructor(root, exportName, depsMap, factoryFn ) {
    var depPathsRequireds = Object.keys(depsMap).map(function(key) { // NOTE: 企业微信PC端并未支持Object.values
        return depsMap[key];
    });
    var depExportNames = Object.keys( depsMap );
    var getDepsExports = function() {
        return depExportNames.map( function(moduleExportName) { 
            var moduleObj = root[moduleExportName];
            
            if( moduleObj ) {
                return moduleObj
            }
            throw new Error('Cannot find module \'' + moduleExportName + '\'');
         });
    }

    if (typeof module === 'object' && typeof module.exports === 'object') { // commonjs模块规范，nodejs环境
        module.exports = factoryFn.apply(null, depPathsRequireds);
    } else if (typeof define === 'function' && define.amd) { // AMD模块规范，require.js
        define(depPaths, factoryFn);
    } else if (typeof define === 'function' && define.cmd) {  // CMD模块规范，sea.js
        define(function(require, exports, module) {
            module.exports = factoryFn.apply(null, depPathsRequireds);
        })
    } else {
        root[exportName] = factoryFn.apply(null, getDepsExports()); // 没有模块环境，直接挂载在全局对象上
    }
}