
UMDConstructor(
    this,
    'OAuth2TokenGuard', 
    {
        'TokenUtils': typeof require !== 'undefined' && require('./token-utils/token.es5'),
        'UrlParamsUtils': typeof require !== 'undefined' && require('./url-params-utils/url-params.es5'),
        'OAuth2CodeGuard': typeof require !== 'undefined' && require('../utils.oauth2-code-guard/oauth2-code-guard.es5'),
    }, 
    function(TokenUtils, UrlParamsUtils, OAuth2CodeGuard) {
        function _findValidTokenOrReLoginAndCacheNewInfoWhenURLChange() { // 页面hash变化时，有必要时也会检查token是否有效，会执行此公用方法
            // NOTE: URL经过转换先不考虑
            // 第三方登录URL跳转应该会导致重载，先不考虑

            var loginInfo = TokenUtils.getLoginInfoFromCache();
            var essentialParamsURLObj = UrlParamsUtils.getEssentialParamsURLObjByMerge( loginInfo );
            var navInfoObj = UrlParamsUtils.getNavInfoObj( essentialParamsURLObj );
        
            TokenUtils.reLoginIfNeed(essentialParamsURLObj, navInfoObj, function(navInfoObj, oauth2CodeAndTargetInfo) {
                // UrlParamsUtils.redirectToFetchOauth2Code(navInfoObj, oauth2CodeAndTargetInfo);
                UrlParamsUtils.cacheCurPendingJumpUrl();
                OAuth2CodeGuard.autoLogonByLoginType( navInfoObj );
            });

            TokenUtils.setLoginInfoFromUrl( essentialParamsURLObj, navInfoObj, loginInfo );

            UrlParamsUtils.setReLoginUrl( essentialParamsURLObj, navInfoObj );
        };

        function run() {
            UrlParamsUtils.recordComInStatesOnInitialization();

            UrlParamsUtils.removeIndexHtmlInUrlPath();

            UrlParamsUtils.checkShouldLoadPendingJumpUrl();

            _findValidTokenOrReLoginAndCacheNewInfoWhenURLChange();
        }

        var OAuth2TokenGuard = {
            run: run,
            _findValidTokenOrReLoginAndCacheNewInfoWhenURLChange: _findValidTokenOrReLoginAndCacheNewInfoWhenURLChange,
        };

        return OAuth2TokenGuard;
    }
);


// 支持模块化引用
function UMDConstructor(root, exportName, depsMap, factoryFn ) {
    var depPathsRequireds = Object.keys(depsMap).map(function(key) { // NOTE: 企业微信PC端并未支持Object.values
        return depsMap[key];
    });
    var depExportNames = Object.keys( depsMap );
    var getDepsExports = function() {
        return depExportNames.map( function(moduleExportName) { 
            var moduleObj = root[moduleExportName];
            
            if( moduleObj ) {
                return moduleObj
            }
            throw new Error('Cannot find module \'' + moduleExportName + '\'');
         });
    }

    if (typeof module === 'object' && typeof module.exports === 'object') { // commonjs模块规范，nodejs环境
        module.exports = factoryFn.apply(null, depPathsRequireds);
    } else if (typeof define === 'function' && define.amd) { // AMD模块规范，require.js
        define(depPaths, factoryFn);
    } else if (typeof define === 'function' && define.cmd) {  // CMD模块规范，sea.js
        define(function(require, exports, module) {
            module.exports = factoryFn.apply(null, depPathsRequireds);
        })
    } else {
        root[exportName] = factoryFn.apply(null, getDepsExports()); // 没有模块环境，直接挂载在全局对象上
    }
}