
UMDConstructor(
    this,
    'TokenUtils',
    {
        'UrlParamsUtils': typeof require !== 'undefined' && require('../url-params-utils/url-params.es5')
    },
    function( UrlParamsUtils ) {
        function clearCurToken( loggedFlagObj ) {
            localStorage.setItem('access_token', ''); // 需要清空，否则后续请求带上token就导致报错
            // localStorage.removeItem('webApi');
            loggedFlagObj.token = null;
            loggedFlagObj.webApi = null;
        }
    
        function checkIsTempUser() {
            return localStorage.getItem('_isTempUser');
        }
    
        /** 
         *  清理或恢复token
         * 
        */
        function findValidTokenIfCan(mergedLoginInfo, loggedFlagObj) {
            var curSharedPatientOrTempUserOnlyKey = checkIsTempUser();
            var newSharedPatientOrTempUserOnlyKey = getSharedPatientOrTempUserOnlyKey(mergedLoginInfo.navProjectId, mergedLoginInfo.target_code); // 分享链接必带projectId
            
            if( mergedLoginInfo.target_code 
                && curSharedPatientOrTempUserOnlyKey != newSharedPatientOrTempUserOnlyKey 
                && curSharedPatientOrTempUserOnlyKey !== '' ) { // 尝试读取临时token
                if( tryRestoreTempToken( newSharedPatientOrTempUserOnlyKey ) ) {
                    curSharedPatientOrTempUserOnlyKey = checkIsTempUser();
                }
            }
    
            // NOTE: 项目不同，需要重新登录，除非是是企业内部应用，由于nbqywx入口没带projectId，不需要清token
            // NOTE: pageDefaultLoginTypeAndProjectIdAsEntry：
            // 即使企业微信默认登录页没有传projectid并且非当前项目id，也会因为进入webapp后，因为请求401退出而重新登录一次
            if( mergedLoginInfo.navProjectId && (loggedFlagObj.curProjId != mergedLoginInfo.navProjectId ) ) { 
                clearCurToken( loggedFlagObj );
            }
    
            if( mergedLoginInfo.target_code ) { // 进入分享页面链接 isRedirectFromShare
                if( curSharedPatientOrTempUserOnlyKey === '' ) { // null 是未设置字段，为''表示真实设置过且为非临时用户
    
                } else if( newSharedPatientOrTempUserOnlyKey === curSharedPatientOrTempUserOnlyKey ) { // 判断是否可复用分享验证码已登录token
                    
                } else {  // 一律进入登录或刷新API、用户配置 的流程
                    clearCurToken( loggedFlagObj );
                }
            } else if( curSharedPatientOrTempUserOnlyKey ) { // 不能用临时token
                clearCurToken( loggedFlagObj );
            }
        }
    
        // 尝试恢复临时token
        function tryRestoreTempToken( storeKey ) {
            var tempTokenKeeper = JSON.parse( localStorage.getItem('_tempTokenKeeper') ) || {};
    
            if( tempTokenKeeper[storeKey] ) {
                var tempUserStorageInfo = tempTokenKeeper[storeKey]
                var tokenInfo = tempUserStorageInfo.loginTokenInfo;
    
                if( tokenInfo ) {
                    localStorage.setItem("token_type", tokenInfo.token_type);
                    localStorage.setItem('isLogin', 'true');
                    localStorage.setItem("access_token", tokenInfo.access_token);
                    localStorage.setItem("loginCode", tokenInfo.logincode);
                    localStorage.setItem("userId", tokenInfo.userId);
                    localStorage.setItem("roles", tokenInfo.roles);
                    localStorage.setItem("MDTRole", tokenInfo.MDTRole);
                    
                    var storageObj = Object.assign( {}, tempUserStorageInfo);
                    delete storageObj.expiredTime;
                    delete storageObj.loginTokenInfo;
                    restoreTempToStorage( storageObj );
                    return tempUserStorageInfo;
                }
            }
            return null;
        }
    
        // 恢复到localStorage
        function restoreTempToStorage( restorObj ) {
            var keys = Object.keys( restorObj );
            for( var keyIndex in keys ) {
                var keyStr = keys[ keyIndex ];
                localStorage.setItem( keyStr, JSON.stringify(restorObj[keyStr]) );
            }
        }
    
        function setLoginInfo( loginInfo ) {
            var oldLoginInfo = getLoginInfoFromCache();
            Object.assign( oldLoginInfo, loginInfo);
            localStorage.setItem('loginInfo', JSON.stringify( oldLoginInfo ));
        }

        function updateLoginInfoByObj( newLoginInfo ) {
            setLoginInfo( UrlParamsUtils.getURLParamsIfCan( newLoginInfo ) || newLoginInfo );
        }
    
        function setLoginInfoFromUrl( urlObj, mergedLoginInfo, loginInfo) {
            var params = UrlParamsUtils.getURLObjParams( urlObj );
    
            // 保证有登录类型
            Object.assign(loginInfo, params, {   // 必需缓存的信息
                projectid: mergedLoginInfo.navProjectId,
                appid: mergedLoginInfo.appid,
                logintype: mergedLoginInfo.logintype
            });
            setLoginInfo(loginInfo);
        }
    
        function getCacheOAuth2UserInfo() {
            var _wxgzhUnionidInfo = localStorage.getItem('_wxgzhUnionidInfo');
            return _wxgzhUnionidInfo;
        }
    
        function isLogged() {
            return TokenUtils.loggedFlagObj.token && TokenUtils.loggedFlagObj.webApi;
        }
        
        function readLoggedInfoFromCache() {
            var loggedFlagObj = {
                webApi: localStorage.getItem('webApi'),   // 动态读取
                token: localStorage.getItem('access_token'),   // 动态读取
            };
            loggedFlagObj.curProjId = (JSON.parse( loggedFlagObj.webApi ) || {}).projectId; // 动态读取
            
            TokenUtils.loggedFlagObj = loggedFlagObj;
    
            return TokenUtils.loggedFlagObj;
        }
    
        function getLoginInfoFromCache() {
            return JSON.parse( localStorage.getItem('loginInfo') ) || {}
        }
    
        // _OAuth2_code 一次有效, 下次index.html进入只需要projectId、loginType【可能刷新的情况】
        function clearDisposableLoginInfoField() {
            var loginInfo = getLoginInfoFromCache();
            delete loginInfo.code;
            delete loginInfo.state;
            localStorage.setItem('loginInfo', JSON.stringify( loginInfo ) );
        }
    
        // target_code、_redirect_from 登录成功跳转一次有效, 下次index.html进入只需要projectId、loginType【可能刷新的情况】s
        function clearDisposableFieldBeforeJump() {
            var loginInfo = getLoginInfoFromCache();
            delete loginInfo.target_code;
            delete loginInfo.state;
            localStorage.setItem('loginInfo', JSON.stringify( loginInfo ) );
        }
    
        /** 
         * 是否需要跳转获取oauth2的code
         * 
        */
         function reLoginIfNeed(urlObj, mergedLoginInfo, fetchOAuth2CodeCallbackFn) {
            var _wxgzhUnionidInfo = getCacheOAuth2UserInfo();
            var oauth2CodeAndTargetInfo = UrlParamsUtils.getCodeForOAuth2UserInfoAndTargetIdByURLObj( urlObj );
            var loggedFlagObj = readLoggedInfoFromCache();
            findValidTokenIfCan(mergedLoginInfo, loggedFlagObj);
            var hasLogged = isLogged(); // NOTE：重新判断
    
            switch( mergedLoginInfo.logintype ) {
                case 'nbqywx':
                case 'dkfqywx':
                    if( !oauth2CodeAndTargetInfo.code && !hasLogged ) {
                        fetchOAuth2CodeCallbackFn && fetchOAuth2CodeCallbackFn(mergedLoginInfo, oauth2CodeAndTargetInfo);
                    }
                    break;
                case 'yywxgzh':
                    if( !oauth2CodeAndTargetInfo.code && !_wxgzhUnionidInfo && !hasLogged ) {
                        fetchOAuth2CodeCallbackFn && fetchOAuth2CodeCallbackFn(mergedLoginInfo, oauth2CodeAndTargetInfo);
                    }
                    break;
            }

        }

        function getSharedPatientOrTempUserOnlyKey( projectId, target_code ) {
            return projectId + '@' + target_code;
        }

        // NOTE: 只有直接访问OAuth2Login页面，未指定登录方式，且存在缓存,才需要修订
        function reviseLoginTypeOnFetchOAuth2CodePageDefaultConflict( logintype, pageDefaultLoginType) {
            if( logintype === 'ThirdPartyPlatform' ) {  // NOTE: 临时方案，后边再修改成配置为指定登录方式; 深大互联网小程序，第三方调阅存在冲突
                logintype = pageDefaultLoginType;
                localStorage.clear();
            }
            return logintype;
        }
        
        var TokenUtils = {
            loggedFlagObj: {},
            clearCurToken: clearCurToken,
            findValidTokenIfCan: findValidTokenIfCan,
            restoreTempToStorage: restoreTempToStorage,
            tryRestoreTempToken: tryRestoreTempToken,
            setLoginInfo: setLoginInfo,
            setLoginInfoFromUrl: setLoginInfoFromUrl,
            getCacheOAuth2UserInfo: getCacheOAuth2UserInfo,
            isLogged: isLogged,
            readLoggedInfoFromCache: readLoggedInfoFromCache,
            clearDisposableLoginInfoField: clearDisposableLoginInfoField,
            clearDisposableFieldBeforeJump: clearDisposableFieldBeforeJump,
            checkIsTempUser: checkIsTempUser,
            getLoginInfoFromCache: getLoginInfoFromCache,
            reLoginIfNeed: reLoginIfNeed,
            updateLoginInfoByObj: updateLoginInfoByObj,
            getSharedPatientOrTempUserOnlyKey: getSharedPatientOrTempUserOnlyKey,
            reviseLoginTypeOnFetchOAuth2CodePageDefaultConflict: reviseLoginTypeOnFetchOAuth2CodePageDefaultConflict
        };

        return TokenUtils;
    }
);


// 支持模块化引用
function UMDConstructor(root, exportName, depsMap, factoryFn ) {
    var depPathsRequireds = Object.keys(depsMap).map(function(key) { // NOTE: 企业微信PC端并未支持Object.values
        return depsMap[key];
    });
    var depExportNames = Object.keys( depsMap );
    var getDepsExports = function() {
        return depExportNames.map( function(moduleExportName) { 
            var moduleObj = root[moduleExportName];
            
            if( moduleObj ) {
                return moduleObj
            }
            throw new Error('Cannot find module \'' + moduleExportName + '\'');
         });
    }

    if (typeof module === 'object' && typeof module.exports === 'object') { // commonjs模块规范，nodejs环境
        module.exports = factoryFn.apply(null, depPathsRequireds);
    } else if (typeof define === 'function' && define.amd) { // AMD模块规范，require.js
        define(depPaths, factoryFn);
    } else if (typeof define === 'function' && define.cmd) {  // CMD模块规范，sea.js
        define(function(require, exports, module) {
            module.exports = factoryFn.apply(null, depPathsRequireds);
        })
    } else {
        root[exportName] = factoryFn.apply(null, getDepsExports()); // 没有模块环境，直接挂载在全局对象上
    }
}







