UMDConstructor(
    this,
    'UrlParamsUtils', 
    {
        'RouteUtils': typeof require !== 'undefined' && require('../../route.utils/route.utils.es5')
    }, 
    function(RouteUtils) {
        // NOTE: 一般url参数不要有与code、state重名的参数，否则与OAuth2常见返回参数冲突

        function getEssentialParamsURLObjByMerge( loginInfo ) {
            var urlObj = new PolyFillURL(location.href);

            var routeEssentialParamsObj = RouteUtils.getRouteEssentialParamsObj();
            
            /** 
             *  补齐参数
             * 
            */
            if( urlObj.href.indexOf( '/social-shares' ) !== -1 ) { // NOTE: 解析出参数用以寻找临时token
                // NOTE: 前边已转成： /no-guard.projectid.appid.logintype/social-shares/:target_code/:_redirect_from
                var paramVals = urlObj.href.split('/social-shares')[1].split('/');
                urlObj.searchParams.set('target_code', paramVals[1]);
                urlObj.searchParams.set('state', paramVals[2]);
            }
            
            if(  routeEssentialParamsObj  ) { // NOTE: 都已转换
                urlObj.searchParams.set('projectid', routeEssentialParamsObj.projectid );
                urlObj.searchParams.set('appid', routeEssentialParamsObj.appid );
                urlObj.searchParams.set('logintype', routeEssentialParamsObj.logintype );
            } else {
                urlObj.searchParams.set('projectid', getProperlyVal( loginInfo.projectid ) );
                urlObj.searchParams.set('appid', getProperlyVal( loginInfo.appid ) );
                urlObj.searchParams.set('logintype', getProperlyVal( loginInfo.logintype ));
            }


            if( !urlObj.searchParams.get('code') ) { // 从缓存填充当前url
                urlObj.searchParams.set('code', loginInfo.code || '');
            }

            if( !urlObj.searchParams.get('state') ) { //  从缓存填充当前url
                urlObj.searchParams.set('state', loginInfo.state || ''); 
            }

            return urlObj;
        }

        // 保证可以重新登录
        function setReLoginUrl( urlObj, navInfoObj ) {
            var _reLoginUrlObj = new PolyFillURL( urlObj.href );

            if( UrlParamsUtils.isLoggedWithinTargetOnly ) {

            } else {
                _reLoginUrlObj.hash = '';
                _reLoginUrlObj.pathname = '/tabs.' + navInfoObj.navProjectId + '.' + navInfoObj.appid + '.' + navInfoObj.logintype + '/home';
                _reLoginUrlObj.search = ''; // NOTE: 清空searchParams
                // _reLoginUrlObj.searchParams.delete('code');  // 去掉
                // _reLoginUrlObj.searchParams.delete('state'); // 不需要指定分享验证码去重新登录
                // _reLoginUrlObj.searchParams.delete('target_code'); // 不需要保留预进页面
            }
            localStorage.setItem('_OAuth2ReLoginUrl', _reLoginUrlObj.href); // 更新重登录url
        }

        function cacheCurPendingJumpUrl() {
            setPendingJumpUrl( location.href );
        }

        function redirectToFetchOauth2Code(navInfoObj, oauth2CodeAndTargetInfo) {
        
            cacheCurPendingJumpUrl(); // 先缓存，回来后再还原
            var hashStr = '';

            // if( location.pathname.indexOf('/app-shares') === 0 ) { // NOTE: 在分享独立页面中
            //     hashStr = '#app-shares';
            // }
            var OAuth2CodeLoginUrl = location.protocol + '//' + location.host + '/OAuth2Login.html';

            window['loc_redirect'](OAuth2CodeLoginUrl
              + '?projectid=' + navInfoObj.navProjectId
              + '&appid=' + navInfoObj.appid 
              + '&logintype=' + navInfoObj.logintype
              + '&target_code=' + oauth2CodeAndTargetInfo.target_code  // 暂时两边都保留清理无效token的功能
              + '&state=' + oauth2CodeAndTargetInfo.state
              + hashStr);
        };

        function isAppDefaultEntry() {
            return location.pathname === '/' 
            || location.pathname === '/index.html' 
            || location.pathname === ''
            || location.pathname === '/app-shares/index.html' ;
        }

        function thirPartyJumpParamsTransform() {
            var rawUrlObj = new PolyFillURL( location.href );
            var projectid = rawUrlObj.searchParams.get('projectid') || rawUrlObj.searchParams.get('projectId');
            var appid = rawUrlObj.searchParams.get('appid');
            
            history.replaceState(null, document.title, UrlParamsUtils._curAppLink 
                + '/no-guard.' + projectid + '.' + appid + '.' +  'ThirdPartyPlatform'
                + '/platform-login'
                + rawUrlObj.search ); // NOTE: 不定参数
        }

        function getProperlyVal( val ) {
            if( val === 'undefined' || val === 'null') {
                return '';
            }
            return val || '';
        }

        // NOTE: 默认项目(不含app-sheares子页)：去掉/index.html；不去匹配路由
        function removeIndexHtmlInUrlPath() {
            if( location.pathname.indexOf('/index.html') === 0 ) {
                var indexDetectUrlObj = new PolyFillURL( location.href );
                indexDetectUrlObj.pathname = indexDetectUrlObj.pathname.replace(/^\/index\.html/, '');
                history.replaceState(null, document.title, indexDetectUrlObj.href);
            }
        }

        // 无需检查登录状态的url模式
        // '#shares/' 与 '#PlatformLogin/'访问有自己的登录流程，非共用，因此可以路由直接访问
        function isNoGuardUrlPattern() {
            return location.href.indexOf('/platform-login/') !== -1
                || location.href.indexOf('/social-shares/') !== -1
                || isThirPartyLoginUrl()
        }

        function isThirPartyLoginUrl() {
            return location.href.indexOf('/PlatformLogin?') !== -1
        }

        function removePendingJumpUrl() {
            localStorage.removeItem('target_url'); 
        }

        function getPendingJumpUrl() {
            var target_url = localStorage.getItem('target_url');
            var target_url_time = localStorage.getItem('target_url_time');

            if( target_url_time && (( new Date().getTime() - target_url_time ) > 36 * 1000) ) { // 
                removePendingJumpUrl();
                return null;
            }
            return target_url;
        }

        function setPendingJumpUrl( url ) {
            localStorage.setItem('target_url_time', new Date().getTime() );
            return localStorage.setItem('target_url', url );
        }

        function checkShouldLoadPendingJumpUrl( ) {
            if( isThirPartyLoginUrl() ) { // NOTE: 第三方登录
                thirPartyJumpParamsTransform();
            } else if( UrlParamsUtils._cache_target_url && isAppDefaultEntry() ) { // NOtE: 访问目标页、一次有效、
                replaceStateToJumpUrl( UrlParamsUtils._cache_target_url );
                removePendingJumpUrl();
            } else if( UrlParamsUtils._cache_target_url ) { // NOTE: 也防止OAuth2Login跳不回来，遗留target_url 
                removePendingJumpUrl();
            }
        }

        function replaceStateToJumpUrl( url ) {
            history.replaceState(null, document.title, url);  // NOTE: OAuth登录的code或者state通过localStorage传参，而非url，因此可覆盖，源参数也能恢复
        }

        function recordComInStatesOnInitialization() {
            UrlParamsUtils._curAppLink = location.protocol + '//' + location.host;
            UrlParamsUtils._cache_target_url = getPendingJumpUrl();  // NOTE: 获取code前进行记录
            UrlParamsUtils.isLoggedWithinTargetOnly = isNoGuardUrlPattern(); // NOTE: 只能设置指定重登录地址
            console.log( '转换前的href：', location.href);
        }

        function getURLObjParams( urlObj ) {
            var params = {};
            /// @ts-ignore
            urlObj.searchParams.forEach( function(val, keyStr) { // for xxx in searchParams.keys()将存在函数
                params[ keyStr ] = val;
            });
            return params;
        }



        // 交换OAuth2用户信息的信令：code
        function getCodeForOAuth2UserInfoAndTargetIdByURLObj( urlObj ) {
            var target_code = urlObj.searchParams.get('target_code') || '';  // NOTE: 分享患者的查询编码
            var state = urlObj.searchParams.get('state') || '';
            var code = urlObj.searchParams.get('code');

            return {
                target_code: target_code,
                state: state,
                code: code,
            }
        }

        function goLink( url ) {
            // NOTE: 部分微信版本测试出location.replace方法发起会被canceled，https://developers.weixin.qq.com/community/develop/doc/000866de410e18d7db8936f5b5b000
            window['loc_redirect'](url);
            
            setTimeout(function() {
                if( location.href !== url ) {
                  window['loc_h_assign']( url );
                }
            }, 700);
        }

        function getURLParamsIfCan( obj ) {
            var searchParams;
            
            if( obj instanceof URLSearchParams ) {
                searchParams = {};
                /// @ts-ignore
                obj.forEach( function (val, keyStr) {
                    searchParams[ keyStr ] = val || '';
                } )
            };
            return searchParams;
        }

        function setSimpleParamsGetPrototypeToURLObj( urlObj ){
            urlObj.constructor.prototype.getParamVal = function( name ) {
                return this.searchParams.get(name);
            };
        }

        // NOTE: 已确定会通过URL参数传过来； logintype需要，用以显示登录方式（主要接收oauth2 code时）；而projectid不需要与缓存合并。DefaultParamsFromLoginInfo
        function getUrlObjAndCheckPageDefaultLoginTypeConflict(cacheLoginInfo, loginTypeConflictWithDefaultCheckFn ) {
            var urlObj = new PolyFillURL(location.href);
            var navProjectId = urlObj.searchParams.get('projectid') || '';  // || cacheLoginInfo.projectid
            var appid = urlObj.searchParams.get('appid') || ''; // || cacheLoginInfo.appid
            var logintype = urlObj.searchParams.get('logintype') || cacheLoginInfo.logintype || '';
            
            logintype = loginTypeConflictWithDefaultCheckFn( logintype );
           
            var target_code = urlObj.searchParams.get('target_code') || ''; // 首次进入，用来清理无效token  // || cacheLoginInfo.target_code
            var state = urlObj.searchParams.get('state') || ''; // 记录，已有缓存
            var code = urlObj.searchParams.get('code') || '';

            
            urlObj.searchParams.set('projectid', navProjectId);
            urlObj.searchParams.set('appid', appid);
            urlObj.searchParams.set('logintype', logintype);

            urlObj.searchParams.set('target_code', target_code);
            urlObj.searchParams.set('state', state);
            urlObj.searchParams.set('code', code);

            return urlObj;
        }

        function getNavInfoObj( urlObj ) {
            setSimpleParamsGetPrototypeToURLObj( urlObj );
            return {
                navProjectId: urlObj.getParamVal('projectid'),
                projectid: urlObj.getParamVal('projectid'),
                appid: urlObj.getParamVal('appid'),
                logintype: urlObj.getParamVal('logintype'),

                target_code: urlObj.getParamVal('target_code'),
                state: urlObj.getParamVal('state'),
                code: urlObj.getParamVal('code'),
            }
        }

        var UrlParamsUtils = {
            isLoggedWithinTargetOnly: false, // NOTE: 是否限制登录后，访问范围不能超出目标页
            _curAppLink: '',
            _cache_target_url: '',
            getEssentialParamsURLObjByMerge: getEssentialParamsURLObjByMerge,
            setReLoginUrl: setReLoginUrl,
            redirectToFetchOauth2Code: redirectToFetchOauth2Code,
            isAppDefaultEntry: isAppDefaultEntry,
            thirPartyJumpParamsTransform: thirPartyJumpParamsTransform,
            removeIndexHtmlInUrlPath: removeIndexHtmlInUrlPath,
            isNoGuardUrlPattern: isNoGuardUrlPattern,
            isThirPartyLoginUrl: isThirPartyLoginUrl,
            removePendingJumpUrl: removePendingJumpUrl,
            getPendingJumpUrl: getPendingJumpUrl,
            checkShouldLoadPendingJumpUrl: checkShouldLoadPendingJumpUrl,
            recordComInStatesOnInitialization: recordComInStatesOnInitialization,
            getURLObjParams: getURLObjParams,
            getCodeForOAuth2UserInfoAndTargetIdByURLObj: getCodeForOAuth2UserInfoAndTargetIdByURLObj,
            goLink: goLink,
            getURLParamsIfCan: getURLParamsIfCan,
            // setUrlObjDefaultParamsFromLoginInfo: setUrlObjDefaultParamsFromLoginInfo,
            getUrlObjAndCheckPageDefaultLoginTypeConflict: getUrlObjAndCheckPageDefaultLoginTypeConflict,
            getNavInfoObj: getNavInfoObj,
            cacheCurPendingJumpUrl: cacheCurPendingJumpUrl
        };

        return UrlParamsUtils;
    }
);


// 支持模块化引用
function UMDConstructor(root, exportName, depsMap, factoryFn ) {
    var depPathsRequireds = Object.keys(depsMap).map(function(key) { // NOTE: 企业微信PC端并未支持Object.values
        return depsMap[key];
    });
    var depExportNames = Object.keys( depsMap );
    var getDepsExports = function() {
        return depExportNames.map( function(moduleExportName) { 
            var moduleObj = root[moduleExportName];
            
            if( moduleObj ) {
                return moduleObj
            }
            throw new Error('Cannot find module \'' + moduleExportName + '\'');
         });
    }

    if (typeof module === 'object' && typeof module.exports === 'object') { // commonjs模块规范，nodejs环境
        module.exports = factoryFn.apply(null, depPathsRequireds);
    } else if (typeof define === 'function' && define.amd) { // AMD模块规范，require.js
        define(depPaths, factoryFn);
    } else if (typeof define === 'function' && define.cmd) {  // CMD模块规范，sea.js
        define(function(require, exports, module) {
            module.exports = factoryFn.apply(null, depPathsRequireds);
        })
    } else {
        root[exportName] = factoryFn.apply(null, getDepsExports()); // 没有模块环境，直接挂载在全局对象上
    }
}
