cordova.define("cordova-plugin-sharescreen.CordovaPluginSharescreen", function(require, exports, module) {
/*
 * Copyright 2017 Qbix Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 */

var exec = require('cordova/exec');

var PLUGIN_NAME = "CordovaPluginSharescreen"

function trace(text) {
  text = text.trim();
  const now = (window.performance.now() / 1000).toFixed(3);

  console.log(now, text);
}

let startTime = null;
let remoteStream;
let remotePeerConnection;
               
function handleSignal(data) {
    // description["sdp"] = description["sdp"].replace(/(\d)+\.(\d)+\.(\d)+\.(\d)+/, ':::1');
    // description["sdp"] = description["sdp"].replace(/[a-z0-9]+\:[a-z0-9]+\:[a-z0-9]+\:[a-z0-9]+\:[a-z0-9]+\:[a-z0-9]+\:[a-z0-9]+\:[a-z0-9]+/, ':::1');
    trace("remoteSocket Received data " + data);
    const description = JSON.parse(data);
    if(description.hasOwnProperty("candidate") || description.hasOwnProperty("serverUrl")) {
        description["candidate"] = description["sdp"];
        const iceCandidate = new RTCIceCandidate(description);
        console.log("remote ice candidate "+JSON.stringify(iceCandidate));
        remotePeerConnection.addIceCandidate(iceCandidate)
            .then(() => {
                trace(`addIceCandidate success.`);
            }).catch((error) => {
                trace(`failed to add ICE Candidate:\n`+`${error.toString()}.`);
            });
    } else if(description.hasOwnProperty("type") && (description["type"]==="offer" || description["type"]==="OFFER")) {
        //offerReceived
        console.log("remote offer");
        trace('remotePeerConnection setRemoteDescription start.');
        description["type"] = description["type"].toLowerCase();
        description["sdp"] = description["description"]
        remotePeerConnection.setRemoteDescription(description)
            .then(() => {
                console.log("setRemoteDescription success");
            }).catch(() => {
                trace(`Failed to create session description: ${error.toString()}.`);
            });

        trace('remotePeerConnection createAnswer start.');
        remotePeerConnection.createAnswer()
            .then(createdAnswer)
            .catch(()=>{
                trace(`Failed to create session description: ${error.toString()}.`);
            });
    } else if(description.hasOwnProperty("type") && (description["type"]==="answer"|| description["type"]==="ANSWER")) {
        //answreReceived
        console.log("remote answer");
        description["type"] = description["type"].toLowerCase();
        description["sdp"] = description["description"]
        trace('remotePeerConnection setRemoteDescription start.');
        remotePeerConnection.setRemoteDescription(description)
            .then(() => {
                console.log("setRemoteDescription success");
            }).catch(()=>{
                trace(`Failed to create session description: ${error.toString()}.`);
            });
    }
}
               
// Logs answer to offer creation and sets peer connection session descriptions.
function createdAnswer(description) {
    trace(`Answer from remotePeerConnection:\n${description.sdp}.`);

    trace('remotePeerConnection setLocalDescription start.');
    remotePeerConnection.setLocalDescription(description)
        .then(() => {
            console.log("setLocalDescription success");
        }).catch(() => {
            trace(`Failed to create session description: ${error.toString()}.`);
        });


    const newDescription = {
        "description":description.sdp,
        "type":"ANSWER"
    }
    
    exports.sendSignal(newDescription)
}

// function checkAndroidVerEqOrBigger( ver ) {
//     try{
//         return platform.os.family.toLowerCase() == 'android' && 
//         parseFloat(platform.os.version) > ver;
//     } catch( err ) {
//         console.error('platform.os.family:', err );
//     }
    
//     var androidVerMatchs = navigator.userAgent.match(/android\ ([\d]*?)[\.\;]/i);
//     return androidVerMatchs && androidVerMatchs[1] > ver;
// }

function handleAddStreamSuccessEv(streamEv, onStream, getStreamCb) {
    if( streamEv.track.kind === 'video' ) {
        $('video#theVideoForward2Canvas').remove();
        var video = document.createElement('video');
        video.id = 'theVideoForward2Canvas';
        video.style.position = 'fixed';
        video.style.zIndex = '9999';
        video.style.left = '0px';
        video.style.bottom = '0px';
        video.style.width = '100px';
        video.style.display = 'none';
        document.body.appendChild( video );
    var stream = streamEv.streams[0];
        video.srcObject = stream ;
        video.play();


        video.addEventListener('loadedmetadata', function() {
//                      console.log(`Remote video videoWidth: ${this.videoWidth}px,  videoHeight: ${this.videoHeight}px`);
            $('canvas#theCanvasReceiveFromVideo').remove();
            var canvas = document.createElement('canvas');
            canvas.id = 'theCanvasReceiveFromVideo';
            canvas.style.position = 'fixed';
            canvas.style.zIndex = '9999';
            canvas.style.left = '0px';
            canvas.style.bottom = '400px';
            canvas.style.width =  100 + 'px';
            canvas.style.height = 100 + 'px';
            canvas.style.display = 'none';
            document.body.appendChild( canvas );

            new Video2Canvas(video, canvas).play();

          var canvasStream = canvas.captureStream();

//                      streamEv.streams[0] = canvasStream;

          onStream({
            track: canvasStream.getVideoTracks()[0],
              streams: [
                canvasStream
              ]
            });
        });
    }
}

exports.startScreenShare = function(onStream, error) {
    startTime = window.performance.now();

    let servers = {
        //        offerToReceiveVideo: true,
        //        offerToReceiveAudio: true,
        //        "iceServers": [], // {"urls": "stun:stun.l.google.com:19302"}
        "sdpSemantics":"unified-plan",
        //        bundlePlolicy: 'max-compat',
        //        rtcpMuxPolicy: 'negotiate',
        enabledAllNetworkAdapter: true,
    };
    var shareStreamWrapper = {};
    var getStreamCb = function( ev ) {
        onStream( ev );
        shareStreamWrapper.shareStream = ev.streams[0];
    }
    
    var pc_constraints = {};
    remotePeerConnection = new RTCPeerConnection(servers,pc_constraints);
    trace('Created remote peer connection object remotePeerConnection.');

      remotePeerConnection.addEventListener('signalingstatechange', function(event){
          const peerConnection = event.target;
          console.log('onsignalingstatechange: ', event);
          trace(`onsignalingstatechange state: ` +`${peerConnection.signalingState}.`);
      });
      remotePeerConnection.addEventListener('connectionstatechange', function(event){
          const peerConnection = event.target;
          console.log('onconnectionstatechange: ', event);
          trace(`onconnectionstatechange state: ` +`${peerConnection.connectionState}.`);
      });
      remotePeerConnection.addEventListener('icecandidate', function(event) {
          const peerConnection = event.target;
          const iceCandidate = event.candidate;
          if (iceCandidate) {
            const newIceCandidate = new RTCIceCandidate(iceCandidate);
            exports.sendSignal(newIceCandidate)

            trace(`ICE candidate:\n` +`${event.candidate.candidate}.`);
          }
      });
      remotePeerConnection.addEventListener('iceconnectionstatechange', function (event) {
          const peerConnection = event.target;
          console.log('ICE state change event: ', event);
          trace(`ICE state: ` +`${peerConnection.iceConnectionState}.`);
      });
      if("ontrack" in remotePeerConnection) {
          remotePeerConnection.addEventListener('track', function (event) {
              console.log("track", event)

            //   var stream = event.streams[0];
            //   stream.getVideoTracks().forEach( item => {
            //       item.addEventListener('ended',function() {
            //           alert('ended');
            //       });
            //       item.addEventListener('mute',function() {
            //           alert('mute');
            //       });
            //       console.log('track state:', item.readyState );
            //   })

            // checkAndroidVerEqOrBigger( 6 )
            if( navigator.userAgent.indexOf('HONOR') !== -1 
                || navigator.userAgent.indexOf('HUAWEI') !== -1) {
                handleAddStreamSuccessEv( event, getStreamCb);
            } else {
                getStreamCb( event );
            }
            
          });
      } else {
          remotePeerConnection.addEventListener('addstream', function (event) {
              console.log("addstream", event)
              getStreamCb(event);
          });
      }

    exec(function(rawData){
        if( rawData == 'BoradcastFinished' ) { // 别处点击结束了屏幕直播
            // error && error( rawData );
            endStream( shareStreamWrapper.shareStream );
            return;
        }
        handleSignal(rawData);
    }, error, PLUGIN_NAME, 'startScreenShare', []);
  
    addDestroyListener( shareStreamWrapper );
};

function endStream( shareStream ) {
    if( shareStream ) {
        shareStream.getTracks().forEach((track) => {
            track.stop();
            track.dispatchEvent(new Event("ended"));
        });
    }
}

added = false;
function addDestroyListener( shareStreamWrapper ) {
    if( !added ) {
        added = true;

        $(window).on("beforeunload", function () { // APP被杀死，停止屏幕分享
            endStream( shareStreamWrapper.shareStream );
        });
    }
}


exports.stopScreenShare = function(success, error) {
    remotePeerConnection.close();
    remotePeerConnection = null;
    exec(success, error, PLUGIN_NAME, 'stopScreenShare', []);
    $('video#theVideoForward2Canvas').remove();
    $('canvas#theCanvasReceiveFromVideo').remove();
}

exports.stopIOSScreenShareBySysUI = function(success, error) {
    exec(success, error, PLUGIN_NAME, 'stopIOSScreenShareBySysUI', []);
}


exports.sendSignal = function(signal,success, error) {
   exec(success, error, PLUGIN_NAME, 'sendSignal', [JSON.stringify(signal)]);
};


// class Video2Canvas {

//     static PLAY_STATUC = { PLAY: 1, PAUSE: 0 }

//     constructor(video, canvas) {
//         this.statuc = Video2Canvas.PLAY_STATUC.PAUSE;
//         this.video = video;
//         if(!(this.canvas = canvas)) {
//             this.canvas = document.createElement("canvas");
//             this.canvas.width = video.videoWidth,
//             this.canvas.height = video.videoHeight,
//             this.canvas.className = video.className;
//             video.style.display = "none"
//             video.parentNode.appendChild(this.canvas);
//         } else {
//             this.canvas.width = video.videoWidth;
//             this.canvas.height = video.videoHeight;
//         }
//         console.log('Video2Canvas width and hight:', this.canvas.width, this.canvas.height );
//         this.c2d = this.canvas.getContext("2d");
//         this.video.addEventListener("play", this.play.bind(this))
//         this.video.addEventListener("pause", this.pause.bind(this))
//     }

//     render(){
//         if(this.statuc == Video2Canvas.PLAY_STATUC.PLAY){
//             this.c2d.clearRect(0, 0, this.canvas.width, this.canvas.height);
//             this.c2d.drawImage(this.video, 0, 0, this.canvas.width, this.canvas.height);
//             requestAnimationFrame(this.render.bind(this))
//         }
//     }

//     play(){
//         console.log("播放")
//         if(this.video.paused) this.video.play();
//         this.statuc = Video2Canvas.PLAY_STATUC.PLAY;
//         this.render();
//         return this;
//     }

//     pause(){
//         console.log("暂停")
//         if(this.video.played) this.video.pause();
//         this.statuc = Video2Canvas.PLAY_STATUC.PAUSE;
//         return this;
//     }
// }


var Video2Canvas = /** @class */ (function () {
    function Video2Canvas(video, canvas) {
        this.statuc = Video2Canvas.PLAY_STATUC.PAUSE;
        this.video = video;
        if (!(this.canvas = canvas)) {
            this.canvas = document.createElement("canvas");
            this.canvas.width = video.videoWidth,
                this.canvas.height = video.videoHeight,
                this.canvas.className = video.className;
            video.style.display = "none";
            video.parentNode.appendChild(this.canvas);
        }
        else {
            this.canvas.width = video.videoWidth;
            this.canvas.height = video.videoHeight;
        }
        console.log('Video2Canvas width and hight:', this.canvas.width, this.canvas.height);
        this.c2d = this.canvas.getContext("2d");
        this.video.addEventListener("play", this.play.bind(this));
        this.video.addEventListener("pause", this.pause.bind(this));
    }
    Video2Canvas.prototype.render = function () {
        if (this.statuc == Video2Canvas.PLAY_STATUC.PLAY) {
            this.c2d.clearRect(0, 0, this.canvas.width, this.canvas.height);
            this.c2d.drawImage(this.video, 0, 0, this.canvas.width, this.canvas.height);
            requestAnimationFrame(this.render.bind(this));
        }
    };
    Video2Canvas.prototype.play = function () {
        console.log("播放");
        if (this.video.paused)
            this.video.play();
        this.statuc = Video2Canvas.PLAY_STATUC.PLAY;
        this.render();
        return this;
    };
    Video2Canvas.prototype.pause = function () {
        console.log("暂停");
        if (this.video.played)
            this.video.pause();
        this.statuc = Video2Canvas.PLAY_STATUC.PAUSE;
        return this;
    };
    Video2Canvas.PLAY_STATUC = { PLAY: 1, PAUSE: 0 };
    return Video2Canvas;
}());
});
